#
fstprob<-function(y,alpha,kmax,cnt,verb=TRUE){
	t1<-proc.time()
	n<-length(y)
#
#	constant for Fisher consistency
#
	fcnt<-sqrt(2*integrate(fpsidn,0,cnt)[[1]]+2*cnt^2*(1-pnorm(cnt)))
#
#	calculate trend
#
	t<-1:n
	t<-(t-mean(t))/sd(t)
	tt<-cbind(t,t^2,t^3,t^4,t^5,t^6,t^7)
	dim(tt)<-c(n,7)
	tmp<-frobreg(y,tt,cnt,fcnt,verb)
	trend<-tmp[[1]]
	if(verb){plot(tmp[[1]],t="l")}
#
	yy<-y-trend   #	subtract trend
#
	scl<-tmp[[4]] #	scale
#
###     stepwise choice of covariates
#
	tmp<-.Fortran(
		"steptrigrob",
		as.double(yy),
		double(3*n),
		as.integer(n),
		double(n),
		double(n),
		double(n),
		double(2*n),
		as.double(scl),
		as.double(alpha),
		as.integer(kmax),
		as.double(cnt),
		as.double(fcnt),
		as.logical(verb)
		)
	kmax=tmp[[10]]   ### number of covariate paris 
#
#       covariates and p-values in order of selection 
#
	pp<-tmp[[7]]
	dim(pp)<-c(n,2)  
	pp<-pp[1:kmax,]
#
###     check no duplication of covariates
#
	k<-2*kmax
	fr<-pp[,1]
	frr<-fr
	k<-length(fr)
	frr[1]<-fr[1]
	j<-1
	for(i in 2:k){
		if(min(abs(frr[1:j]-fr[i]))>0){
			j<-j+1
			frr[j]<-fr[i]
		}
	}
	k<-j
	fr<-frr[1:k]    # chosen covariates
#
#	robust regression based on all chosen covariates
#
	x<-double(2*k*n)
	dim(x)<-c(n,2*k)
	for(j in 1:k){
		x[,(2*j-1)]<-sin(pi*fr[j]*(1:n)/n)
		x[,(2*j)]<-cos(pi*fr[j]*(1:n)/n)
	}
	tmp<-frobreg(yy,x,cnt,fcnt,verb)
#
#
#
	fn<-tmp[[1]]+trend 	# rehression function
	theta<-tmp[[3]]	   	# coefficient values
	res<-tmp[[2]]      	# residuals
	scl<-tmp[[4]]           # scale
	t2<-proc.time()
	print(t2-t1)
	list(fn,res,theta,scl,fr,trend)
}
#
#
#       robust regression
#
#
frobreg<-function(y,x,cnt,fcnt,scl=-1,verb=TRUE){
	n<-length(y)
	k<-length(x[1,])
	b<-lm(y~x)
	res<-b$res
	sig<-sqrt(mean(res^2))
	sig0<-0
	theta<-b$coef
	while( abs(sig0-sig)>10^-3*sig){
		for(i in 1:n){
			res[i]<-sig*fpsi(res[i]/sig,cnt)
		}
		b<-lm(res~x)
		theta<-theta+1.5*b$coef
		sig0<-sig
		sig<-0
		for(i in 1:n){
			sig<-sig+fpsi(res[i]/sig0,cnt)^2
		}
		sig<-sig0*sqrt(sig/n)/fcnt
		if(verb){print(abs(sig0-sig)/sig)}
		res<-b$res
	}
	dim(theta)<-c(k+1,1)
	fn<-theta[1,1]+x%*%theta[2:(k+1),1]
	res<-y-fn
	fn<-as.double(fn)
	theta<-as.double(theta)
	res<-as.double(res)
	list(fn,res,theta,sig)
}
#
#	Huber psi-function
#
fpsi<-function(x,ctn){
	if(abs(x)<ctn){psi<-x}
	else{psi<-ctn*sign(x)}
	return(psi)
}
#
#	required for calculation of constant for Fisher consistency
#
fpsidn<-function(x){
	u<-x^2*dnorm(x)
	return(u)
}

